@@ -1,22 +1,20 @@
 /*
- * Copyright (c) 2019 Yubico AB. All rights reserved.
+ * Copyright (c) 2019-2021 Yubico AB. All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the LICENSE file.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <assert.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
 #include "mutator_aux.h"
 #include "wiredata_fido2.h"
 #include "dummy.h"
 
-#include "fido.h"
-#include "fido/credman.h"
-
 #include "../openbsd-compat/openbsd-compat.h"
 
 /* Parameter set defining a FIDO2 credential management operation. */
@@ -167,7 +165,7 @@ size_t
 pack_dummy(uint8_t *ptr, size_t len)
 {
 	struct param dummy;
-	uint8_t blob[4096];
+	uint8_t blob[MAXCORPUS];
 	size_t blob_len;
 
 	memset(&dummy, 0, sizeof(dummy));
@@ -207,21 +205,10 @@ static fido_dev_t *
 prepare_dev(void)
 {
 	fido_dev_t *dev;
-	fido_dev_io_t io;
 	bool x;
 
-	memset(&io, 0, sizeof(io));
-
-	io.open = dev_open;
-	io.close = dev_close;
-	io.read = dev_read;
-	io.write = dev_write;
-
-	if ((dev = fido_dev_new()) == NULL || fido_dev_set_io_functions(dev,
-	    &io) != FIDO_OK || fido_dev_open(dev, "nodev") != FIDO_OK) {
-		fido_dev_free(&dev);
+	if ((dev = open_dev(0)) == NULL)
 		return NULL;
-	}
 
 	x = fido_dev_is_fido2(dev);
 	consume(&x, sizeof(x));
@@ -287,10 +274,8 @@ get_rp_list(const struct param *p)
 	for (size_t i = 0; i < fido_credman_rp_count(rp) + 1; i++) {
 		consume(fido_credman_rp_id_hash_ptr(rp, i),
 		    fido_credman_rp_id_hash_len(rp, i));
-		consume(fido_credman_rp_id(rp, i),
-		    xstrlen(fido_credman_rp_id(rp, i)));
-		consume(fido_credman_rp_name(rp, i),
-		    xstrlen(fido_credman_rp_name(rp, i)));
+		consume_str(fido_credman_rp_id(rp, i));
+		consume_str(fido_credman_rp_name(rp, i));
 	}
 
 	fido_credman_rp_free(&rp);
@@ -331,10 +316,8 @@ get_rk_list(const struct param *p)
 		consume(fido_cred_pubkey_ptr(cred), fido_cred_pubkey_len(cred));
 		consume(fido_cred_user_id_ptr(cred),
 		    fido_cred_user_id_len(cred));
-		consume(fido_cred_user_name(cred),
-		    xstrlen(fido_cred_user_name(cred)));
-		consume(fido_cred_display_name(cred),
-		    xstrlen(fido_cred_display_name(cred)));
+		consume_str(fido_cred_user_name(cred));
+		consume_str(fido_cred_display_name(cred));
 		val = fido_cred_prot(cred);
 		consume(&val, sizeof(val));
 	}
@@ -359,10 +342,40 @@ del_rk(const struct param *p)
 	fido_dev_free(&dev);
 }
 
+static void
+set_rk(const struct param *p)
+{
+	fido_dev_t *dev = NULL;
+	fido_cred_t *cred = NULL;
+	const char *pin = p->pin;
+	int r0, r1, r2;
+
+	set_wire_data(p->del_wire_data.body, p->del_wire_data.len);
+
+	if ((dev = prepare_dev()) == NULL)
+		return;
+	if ((cred = fido_cred_new()) == NULL)
+		goto out;
+	r0 = fido_cred_set_id(cred, p->cred_id.body, p->cred_id.len);
+	r1 = fido_cred_set_user(cred, p->cred_id.body, p->cred_id.len, p->rp_id,
+	    NULL, NULL);
+	if (strlen(pin) == 0)
+		pin = NULL;
+	r2 = fido_credman_set_dev_rk(dev, cred, pin);
+	consume(&r0, sizeof(r0));
+	consume(&r1, sizeof(r1));
+	consume(&r2, sizeof(r2));
+out:
+	fido_dev_close(dev);
+	fido_dev_free(&dev);
+	fido_cred_free(&cred);
+}
+
 void
 test(const struct param *p)
 {
 	prng_init((unsigned int)p->seed);
+	fuzz_clock_reset();
 	fido_init(FIDO_DEBUG);
 	fido_set_log_handler(consume_str);
 
@@ -370,6 +383,7 @@ test(const struct param *p)
 	get_rp_list(p);
 	get_rk_list(p);
 	del_rk(p);
+	set_rk(p);
 }
 
 void
